################################################################################
#                                                                              #
#      :::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]:::      #
#    ____                __                                                    #
#   / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __ #
#  / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // / #
# /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/  #
#      /___/ /___/                 /_/    /_/           /___/ /_/              #
#                                                                              #
################################################################################
#                                                                              #
# memo.tcl 1.1                                                                 #
#                                                                              #
# Author: Stream@Rusnet <stream@eggdrop.org.ru>                                #
#                                                                              #
# Official support: irc.eggdrop.org.ru @ #eggdrop                              #
#                                                                              #
################################################################################

namespace eval memo {}

setudef flag nopubmemo
setudef flag nowarnmemo

#################################################

bind pub - !memo	::memo::pub_memo
bind msg - !memo	::memo::msg_memo
bind msg - memo		::memo::msg_readmemo

bind join - *		::memo::onjoin
bind nick - *		::memo::onnick
bind time "01 * * * *"	::memo::check_expire

#################################################

catch {unset memo}
foreach p [array names memo *] {catch {unset memo($p)}}

# max number of memos for 1 nick
set memo(max)		10
# delete memos after .. days
set memo(expire)	7
# delay before memo warning (seconds)
set memo(delay)		5
# filename to store memos
set memo(datafile)	"memo.data"

#################################################
##### DON'T CHANGE ANYTHING BELOW THIS LINE #####
#################################################

set memo(data)		""

set memo(ver) 		"1.1"
set memo(authors)	"Stream@Rusnet <stream@eggdrop.org.ru>"

#################################################

proc ::memo::tolower {text} {
	return [string map {                                                                 } [string tolower $text]]
}

proc ::memo::out {nick chan text} {
	if {[validchan $chan]} {putserv "PRIVMSG $chan :$nick, $text"
	} elseif {$nick == $chan} {putserv "PRIVMSG $nick :$text"
	} else {putserv "NOTICE $nick :$text"}
}

proc ::memo::readdata {} {
	global memo
	set memo(data) ""
	if {![catch {set fid [open $memo(datafile) "r"]}]} {
		while {![eof $fid]} {
			set data [split [string trim [gets $fid]] " "]
			if {[llength $data] >= 4} {
				lappend memo(data) [list [lindex $data 0] [lindex $data 1] [lindex $data 2] [lrange $data 3 end]]
			}
		}
		close $fid
		putlog "\[memo\] loaded [llength $memo(data)] memos"
	} else {putlog "\[memo\] can't open file '$memo(datafile)'!"}
}

proc ::memo::writedata {} {
	global memo
	set fid [open "$memo(datafile)" "w+"]
	foreach data $memo(data) {
		puts $fid "[lindex $data 0] [lindex $data 1] [lindex $data 2] [join [lindex $data 3]]  "
	}
	close $fid
}

proc ::memo::init {} {
	::memo::readdata
}

proc ::memo::numstr {val str1 str2 str3} {
	set d1 [expr $val % 10]
	set d2 [expr $val % 100]
	if {$d2 < 10 || $d2 > 19} {
		if {$d1 == 1} {return $str2}
		if {$d1 >= 2 && $d1 <= 4} {return $str3}
	}
	return $str1
}

#################################################

proc ::memo::pub_memo {nick uhost hand chan args} {
	if {[channel get $chan nopubmemo]} {return}
	::memo::addmemo $nick $chan [string trim [join $args]]
}

proc ::memo::msg_memo {nick uhost hand args} {
	::memo::addmemo $nick $nick [string trim [join $args]]
}

proc ::memo::msg_readmemo {nick uhost hand args} {
	set args [split [join $args] " "]
	set res [::memo::getmemos $nick]
	if {[llength $res] < 1} {::memo::out $nick $nick "   ."; return}
	if {[llength $args] < 2} {
		::memo::out $nick $nick "  \002[llength $res]\002 [::memo::numstr [llength $res] "" "" ""]."
		::memo::out $nick $nick " \002memo <read|del> <_|all>\002"
		return
	}
	set act [lindex $args 0]
	set num [lindex $args 1]
	if {![string equal $act "read"] && ![string equal $act "del"]} {
		::memo::out $nick $nick " \002memo <read|del> <_|all>\002"
		return
	}
	if {![regexp -nocase -- {^[0-9]+$} $num] && ![string equal $num "all"]} {
		::memo::out $nick $nick "   ."
		return
	}
	if {![string equal $num "all"] && $num > [llength $res]} {
		::memo::out $nick $nick "   \002[llength $res]\002 [::memo::numstr [llength $res] "" "" ""]."
		return
	}
	if {[string equal $act "read"]} {::memo::readmemo $nick $num $res} else {::memo::delmemo $nick $num}
}

proc ::memo::onjoin {nick uhost hand chan} {
	global botnick
	if {![string equal $nick $botnick]} {::memo::delayed_warn $nick $chan}
}

proc ::memo::onnick {nick uhost hand chan newnick} {
	global botnick
	if {![string equal $nick $botnick]} {::memo::delayed_warn $newnick $chan}
}

proc ::memo::check_expire {min hour day month year} {
	global memo
	set res ""
	set expireseconds [expr {$memo(expire) * 86400}]
	foreach data $memo(data) {
		if {[expr {[unixtime] - [lindex $data 2]}] < $expireseconds} {lappend res $data}
	}
	set memo(data) $res
	::memo::writedata
}

#################################################

proc ::memo::getmemos {nick} {
	global memo
	set nick [::memo::tolower [string trim $nick]]
	set res ""
	foreach data $memo(data) {
		if {[string equal [lindex $data 0] $nick]} {lappend res $data}
	}
	return $res
}

proc ::memo::addmemo {nick chan args} {
	global memo
	set args [split [join $args] " "]
	if {[llength $args] < 2 || [string length [lindex $args 0]] < 1 || [string length [lindex $args 1]] < 1} {
		::memo::out $nick $chan " \002!memo <> <>\002."
		return
	}
	if {[expr {[llength [::memo::getmemos [lindex $args 0]]] + 1}] > $memo(max)} {
		::memo::out $nick $chan "   \002[lindex $args 0]\002   (\002$memo(max)\002)."
		return
	}
	putlog "\[memo\] $nick/$chan [lindex $args 0] [join [lrange $args 1 end]]"
	set data [list [::memo::tolower [lindex $args 0]] $nick [unixtime] [lrange $args 1 end]]
	lappend memo(data) $data
	if {[string equal [::memo::tolower $nick] [::memo::tolower [lindex $args 0]]]} {
		::memo::out $nick $chan "   ."
	} else {::memo::out $nick $chan "  \002[lindex $args 0]\002 ."}
	::memo::writedata
}

proc ::memo::readmemo {nick num res} {
	putlog "\[memo\] $nick read $num"
	set cnum 0
	foreach data $res {
		incr cnum
		if {![string equal $num "all"] && $cnum != $num} {continue}
		::memo::out $nick $nick "$cnum. [join [lindex $data 3]] ([lindex $data 1] [clock format [lindex $data 2] -format "%d.%m.%y %H:%M:%S"])"
	}
}

proc ::memo::delmemo {nick num} {
	global memo
	putlog "\[memo\] $nick delete $num"
	set enick [::memo::tolower [string trim $nick]]
	set cnum 0
	set res ""
	foreach data $memo(data) {
		if {[string equal [lindex $data 0] $enick]} {
			incr cnum
			if {![string equal $num "all"] && $cnum != $num} {lappend res $data}
		} else {lappend res $data}
	}
	if {[llength $res] < [llength $memo(data)]} {
		set memo(data) $res
		if {[string equal $num "all"]} {::memo::out $nick $nick "  ."
		} else {::memo::out $nick $nick "   \002$num\002."}
		::memo::writedata
	}
}

proc ::memo::delayed_warn {nick chan} {
	global memo
	if {![validchan $chan]} {return}
	if {[channel get $chan nowarnmemo]} {return}
	foreach tmr [utimers] {
		set cmd [split [join [lindex $tmr 1]] " "]
		if {[string equal [lindex $cmd 0] "::memo::warn"] && [string equal [lindex $cmd 1] $nick]} {
			killutimer [lindex $tmr 2]
		}
	}
	set tmr [utimer $memo(delay) "::memo::warn {[list $nick]}"]
}

proc ::memo::warn {nick} {
	global botnick
	set nick [join $nick]
	set count [llength [::memo::getmemos $nick]]
	if {$count > 0} {
		::memo::out $nick "" "  \002$count\002 [::memo::numstr $count "" "" ""].  /   \002/msg $botnick memo\002"
	}
}

::memo::init
putlog "memo.tcl v$memo(ver) by $memo(authors) loaded"

